-----------------------------------------------------------------------------------------------------------
--[[
	NaW - Units Manager Class
    Copyright (C) 2011.
	Written by Roland Yonaba - E-mail: roland[dot]yonaba[at]gmail[dot]com
		
		This file Units behaviour
--]]
-----------------------------------------------------------------------------------------------------------

local require  = require
local ipairs = ipairs
local ceil = math.ceil
local floor = math.floor
local abs = math.abs
local insert = table.insert
local remove = table.remove

local CONSTANTS = require 'Scripts/Constants'
local Utils = require 'Scripts/Utils/Unit'
local Astar = require 'Scripts/Libraries/Astar'
local nOfUnits = 0
local pathfinding_queue = {}
local selected_units = {}
local Images = Utils.loadAllUnits('Images/Units/')

local Unit =  { x = 0, y = 0, w = 0, h = 0, health = 0, timer = 1, type = '',
		  move = false, selected = false, speed = 0, state = 'Standing', direction = 1,
		  path = {}, cur = 0, pathLen = 0, temp_x = 0, temp_y = 0,
		  img = ''}

Unit.all = {}

function Unit:new(x, y, utype)
	local u = {}
	self.__index = self
	setmetatable(u,self)
	u:setAbsolutePosition(x or 0, y or 0)	
	u.type = utype
	u.health = self.Parameters[u.type].Health
	u.speed = self.Parameters[u.type].Speed
	u.img = Images[u.type][u.state][u.direction][1]
	u.w = u.img:getWidth()
	u.h = u.img:getHeight()
	insert(self.all,u)
	nOfUnits = nOfUnits + 1
	selected_units[nOfUnits] = false
end
		  

Unit.Parameters = { 
					Villager = {Health = 100, Speed = 0.5},
				  }

function Unit:setAbsolutePosition(x,y)
	self.x = x
	self.y = y
end

function Unit:setMapPosition(map,x,y)
	self.x = map.tileWidth * x - map.tileWidth
	self.y = map.tileHeight * y - map.tileHeight
end

function Unit:isVisible(map)	
	if  (self.health > 0) and ((map.mapX < self.x + self.w) and 
		(map.mapX + map.backgroundWidth > self.x) and 
		(map.mapY < self.y+self.h) and 
		(map.mapY + map.backgroundHeight > self.y)) then
		return true
	end
	return false
end

function Unit:updateAll(delta_time, map, cursor)
	local unit
	for i = 1,nOfUnits do
	unit = self.all[i]
	unit.mX = floor(unit.x / map.tileWidth)+1
	unit.mY = floor(unit.y / map.tileHeight)+1
		local _i
		if unit.state == 'Standing' then
		unit.timer = 1
		_i = unit.timer
		elseif unit.state == 'Moving' then	
		unit.timer = unit.timer + 0.1
			if unit.timer > 3 then unit.timer = 1 end
		_i = floor(unit.timer)
			if unit.x < unit.temp_x then
			unit.direction = 7
			unit.x = unit.x + unit.speed * (delta_time)
			elseif unit.x > unit.temp_x then
			unit.direction = 3
			unit.x = unit.x - unit.speed * (delta_time)
			end
			if unit.y < unit.temp_y then
			unit.direction = 1
			unit.y = unit.y + unit.speed * (delta_time)
			elseif unit.y > unit.temp_y then
			unit.direction = 5
			unit.y = unit.y - unit.speed * (delta_time)
			end			
			if (abs(unit.x - unit.temp_x) <= 2) and (abs(unit.y-unit.temp_y) <= 2) then
				if unit.cur < unit.pathLen then
				unit.cur = unit.cur + 1
				unit.temp_x = unit.path[unit.cur].x * map.tileWidth - map.tileWidth
				unit.temp_y = unit.path[unit.cur].y * map.tileHeight - map.tileHeight
				else
				unit.state = 'Standing'
				unit.timer = 1
				_i = unit.timer
				unit.x = unit.path[unit.pathLen].x * map.tileWidth - map.tileWidth
				unit.y = unit.path[unit.pathLen].y * map.tileHeight - map.tileHeight
				unit.cur = 0
				unit.pathLen = 0
				unit.path = {}
				unit.temp_x,unit.temp_y = 0,0
				end
			end		
		end		
		
		unit.img = Images[unit.type][unit.state][unit.direction][_i]
		
		if unit.selected then
		selected_units[i] = true
		else selected_units[i] = false
		end				
	end
	
	if cursor:pressedOnce('l') then
		for k in ipairs(selected_units) do
			if selected_units[k] then
			local sNode = { x = self.all[k].mX, y = self.all[k].mY }
			local fNode = {x = 0, y = 0}
				if cursor:isStaticHovering(map.minimap) then
				local x,y = map.minimap:revertCoordinates(map, cursor.x, cursor.y)
				x = x + map.backgroundWidth
				y = y + map.backgroundHeight
				fNode = { x = floor(x / map.tileWidth)+1, y = floor(y / map.tileHeight)+1 }
				else fNode = { x = floor((cursor.x + map.mapX) / map.tileWidth)+1, y = floor((cursor.y + map.mapY) / map.tileHeight)+1 }
				end
				if sNode.x ~= fNode.x and sNode.y~=fNode.y then
				insert(pathfinding_queue,{id = k, start = sNode, final = fNode})
				--print('start: x '..sNode.x..' y: '..sNode.y..' End : x '..fNode.x..' y: '..fNode.y)
				end
			end
		end
	end
			
	if #pathfinding_queue > 0 then
	local path = Astar.CalcPath(
									map.cMap, 
									pathfinding_queue[1].start.x, pathfinding_queue[1].start.y, 
									pathfinding_queue[1].final.x, pathfinding_queue[1].final.y, 1, 2 )
									

		if path and #path > 0 then
		local i = pathfinding_queue[1].id
		self.all[i].path = path
		self.all[i].state = 'Moving'
		self.all[i].pathLen = #path
		self.all[i].cur = 1
		self.all[i].temp_x = self.all[i].path[1].x * map.tileWidth - map.tileWidth
		self.all[i].temp_y = self.all[i].path[1].y * map.tileHeight - map.tileHeight
		end
	remove(pathfinding_queue,1)	
	end
end

function Unit:drawAll(map,cursor)
	local unit
	for i = 1,nOfUnits do	
	unit = self.all[i]
		if cursor:dragging(unit) then
		unit.selected = true
		end
		if unit:isVisible(map) then
			if cursor:isHovering(unit, map.mapX, map.mapY) and cursor:isDown('l') then
			unit.selected = true
			end
			if unit.selected then
			love.graphics.setColor(CONSTANTS.COLORS.RED)
			love.graphics.rectangle('fill',unit.x - map.mapX, unit.y - 10 - map.mapY, 30, 2)
			love.graphics.setColor(CONSTANTS.COLORS.GREEN)
			love.graphics.rectangle('fill',unit.x - map.mapX, unit.y - 10 - map.mapY, 30 * ceil(unit.health/self.Parameters[unit.type].Health), 2)
			love.graphics.setColor(CONSTANTS.COLORS.WHITE)			
			end
			love.graphics.draw(unit.img, unit.x - map.mapX, unit.y-map.mapY)
		end
	end
end

function Unit:deselectAll(button)
	if button == 'r' then	
		for k = 1,nOfUnits do 
		self.all[k].selected = false
		end
	end
end

return setmetatable(Unit, {__call = function(self,...) return self:new(...) end} )